#include "SuperNovaDDT/clusterInfo.h"
#include "DDTBaseDataProducts/HitList.h"
#include <vector>

namespace novaddt{
  class SliceSelector;
}

template <class T>
class Limits{
    public: 
        Limits():fHasMin(false),fHasMax(false){}

        Limits(fhicl::ParameterSet const & p):
            fHasMin(p.get_if_present<T>("min",fMin)), 
            fHasMax(p.get_if_present<T>("max",fMax))
        {}
        bool Contains (const T& v) const {
            return (!fHasMin || fMin<=v)&&(!fHasMax || v<=fMax);
        }

    template <class U>
    friend std::ostream& operator << (std::ostream& s,const Limits<U>& lims) {
        s<<"[";
        if(lims.fHasMin) s<<lims.fMin<<"<";
        s<<"x";
        if(lims.fHasMax) s<<"<"<<lims.fMax;
        s<<"]";
        return s;
    }

    private:
        T fMin,fMax;
        bool fHasMin = false;
        bool fHasMax = false;
};

template <class T> 
Limits<T> makeLimits(const fhicl::ParameterSet& p){
    return Limits<T>(p);
}

class novaddt::SliceSelector{
/// a class to check if the slice passes given cut
  public:
   SliceSelector(fhicl::ParameterSet const & p)
   {
        p.get_if_present<Limits<uint64_t>, fhicl::ParameterSet>
            ("ADC",lim_adc, makeLimits<uint64_t>);
        p.get_if_present<Limits<uint64_t>, fhicl::ParameterSet>
            ("nHits.View",lim_Nview, makeLimits<uint64_t>);
        p.get_if_present<Limits<uint64_t>, fhicl::ParameterSet>
            ("nHits.Total",lim_Ntot, makeLimits<uint64_t>);
        p.get_if_present<Limits<uint64_t>, fhicl::ParameterSet>
            ("FV.x",lim_x, makeLimits<uint64_t>);
        p.get_if_present<Limits<uint64_t>, fhicl::ParameterSet>
            ("FV.y",lim_y, makeLimits<uint64_t>);
        p.get_if_present<Limits<uint64_t>, fhicl::ParameterSet>
            ("FV.z",lim_z, makeLimits<uint64_t>);
   }
        
   bool checkSlice(const novaddt::HitList& slice)
   {
       clusterInfo c=CalcCluster(slice); 
       return checkCluster(c);
   }
       //check if adc value for slice is whithin limits
   bool checkCluster(const clusterInfo& c)
   {
        return  lim_adc.Contains(c.ADC()) &&
                lim_Nview.Contains(c.Nx()) &&
                lim_Nview.Contains(c.Ny()) &&
                lim_Ntot.Contains(c.N()) &&
                lim_x.Contains(c.x()) &&
                lim_y.Contains(c.y()) &&
                lim_z.Contains(c.z())
                ;
   }
   size_t countPassing(const std::vector<novaddt::HitList>&slices)
   {
        size_t nslices=0;
        for(const auto& slice: slices)
            if(checkSlice(slice))++nslices;
        return nslices;
    }
   friend std::ostream& operator << (std::ostream& s,const novaddt::SliceSelector& sel){
         s<<"adc: "<<sel.lim_adc
         <<"\tNview: "<<sel.lim_Nview
         <<"\tNtot: "<<sel.lim_Ntot;
         return s;
    }
  private:
    Limits<uint64_t> lim_adc;
    Limits<uint64_t> lim_Nview, lim_Ntot;
    Limits<uint64_t> lim_x, lim_y, lim_z;

};




