////////////////////////////////////////////////////////////////////////
// Class:       HitsAna
// Module Type: analyzer
// File:        HitsAna_module.cc
//
// Generated at Thu Feb 23 14:44:41 2017 by Andrey Sheshukov using artmod
// from cetpkgsupport v1_10_02.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"
#include "DDTBaseDataProducts/EventHeader.h"	
#include "DDTBaseDataProducts/HitList.h"

#include "art/Framework/Services/Optional/TFileService.h"
#include "TTree.h"

#include <limits>
#include <iomanip>

namespace novaddt {
  class HitsAna;
}

class novaddt::HitsAna : public art::EDAnalyzer {
public:
  explicit HitsAna(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  HitsAna(HitsAna const &) = delete;
  HitsAna(HitsAna &&) = delete;
  HitsAna & operator = (HitsAna const &) = delete;
  HitsAna & operator = (HitsAna &&) = delete;

  void analyze(art::Event const & e) override;
  void clearHits();
  void addHit(const novaddt::DAQHit& );
private:

  // Declare member data here.
  art::InputTag fHitsTag;

  std::vector<novaddt::TDC::value_type> fTime;
  std::vector<novaddt::ADC::value_type> fADC;
  std::vector<novaddt::Plane::value_type> fPlane;
  std::vector<novaddt::Cell::value_type> fCell;
  std::vector<novaddt::View::value_type> fView;
  TTree* fTree;
  novaddt::TDC::value_type fEvtTime;
};


novaddt::HitsAna::HitsAna(fhicl::ParameterSet const & p)
  :
  EDAnalyzer(p), 
  fHitsTag(p.get<std::string>("hits_tag"))
{
    art::ServiceHandle<art::TFileService> tfs;
    fTree=tfs->make<TTree>("Hits","Hits analysis tree");
    fTree->Branch("tdc",&fTime);
    fTree->Branch("adc",&fADC);
    fTree->Branch("plane",&fPlane);
    fTree->Branch("cell",&fCell);
    fTree->Branch("view",&fView);
}

void novaddt::HitsAna::clearHits(){
    fTime.clear();
    fADC.clear();
    fPlane.clear();
    fCell.clear();
    fView.clear();
}

void novaddt::HitsAna::addHit(const novaddt::DAQHit& h){
    fTime.push_back(h.TDC().val-fEvtTime);
    fEvtTime=h.TDC().val;
    fADC.push_back( h.ADC().val);
    fPlane.push_back(h.Plane().val);
    fCell.push_back(h.Cell().val);
    fView.push_back(h.View().val);
}

void novaddt::HitsAna::analyze(art::Event const & e)
{
    auto hits=e.getValidHandle<novaddt::HitList>(fHitsTag);
    clearHits();
    if(!hits->empty()) fEvtTime=hits->front().TDC().val;

    for(auto& h: *hits){
        addHit(h);
    }
    fTree->Fill();
  // Implementation of required member function here.
}

DEFINE_ART_MODULE(novaddt::HitsAna)
